@@ -5,7 +5,7 @@ from streamlit.web import cli as stcli
 def run():
     file_path = os.path.realpath(__file__)
     os.chdir(os.path.dirname(file_path))
-    os.system("python -m streamlit run AlphaPeptStats.py --global.developmentMode=false --server.maxUploadSize 500")
+    os.system("python -m streamlit run AlphaPeptStats.py --global.developmentMode=false")
     _this_file = os.path.abspath(__file__)
     _this_directory = os.path.dirname(_this_file)
 
@@ -21,7 +21,7 @@ def run():
 
     print(f'Starting AlphaPeptStats from {file_path}')
 
-    args = ["streamlit", "run", file_path, "--global.developmentMode=false", "--browser.gatherUsageStats=False", "--server.maxUploadSize 500"]
+    args = ["streamlit", "run", file_path, "--global.developmentMode=false"]
 
     sys.argv = args
 
