@@ -2,6 +2,28 @@ import plotly
 import seaborn as sns
 import plotly.graph_objects as go
 
+
+plotly.io.templates["alphastats_colors"] = plotly.graph_objects.layout.Template(
+    layout=plotly.graph_objects.Layout(
+        paper_bgcolor="rgba(0,0,0,0)",
+        plot_bgcolor="rgba(0,0,0,0)",
+        colorway=[
+            "#009599",
+            "#005358",
+            "#772173",
+            "#B65EAF",  # pink
+            "#A73A00",
+            "#6490C1",
+            "#FF894F",
+            "#2B5E8B",
+            "#A87F32",
+        ],
+    )
+)
+
+plotly.io.templates.default = "simple_white+alphastats_colors"
+
+
 class PlotUtils:
     def __init__(self) -> None:
         pass
@@ -20,7 +42,9 @@ class PlotUtils:
         # convert dict back to plotly figure
         return go.Figure(fig_dict)
 
-    def _update_figure_attributes(self, figure_object, plotting_data, preprocessing_info, method=None):
+    def _update_figure_attributes(
+        self, figure_object, plotting_data, preprocessing_info, method=None
+    ):
         setattr(figure_object, "plotting_data", plotting_data)
         setattr(figure_object, "preprocessing", preprocessing_info)
         setattr(figure_object, "method", method)
