@@ -5,9 +5,28 @@ import numpy as np
 import pandas as pd
 import plotly.express as px
 import plotly.graph_objects as go
-
+import plotly
 from functools import lru_cache
 
+plotly.io.templates["alphastats_colors"] = plotly.graph_objects.layout.Template(
+    layout=plotly.graph_objects.Layout(
+        paper_bgcolor="rgba(0,0,0,0)",
+        plot_bgcolor="rgba(0,0,0,0)",
+        colorway=[
+            "#009599",
+            "#005358",
+            "#772173",
+            "#B65EAF",  # pink
+            "#A73A00",
+            "#6490C1",
+            "#FF894F",
+            "#2B5E8B",
+            "#A87F32",
+        ],
+    )
+)
+
+plotly.io.templates.default = "simple_white+alphastats_colors"
 
 class VolcanoPlot(PlotUtils):
     def __init__(
@@ -15,7 +34,8 @@ class VolcanoPlot(PlotUtils):
         column=None, method=None, 
         labels=None, min_fc=None, 
         alpha=None, draw_line=None, 
-        plot=True, perm=100, fdr=0.05
+        plot=True, perm=100, fdr=0.05,
+        color_list=[]
     ):  
         self.dataset = dataset
         self.group1 = group1
@@ -31,8 +51,9 @@ class VolcanoPlot(PlotUtils):
         self.res = None
         self.pvalue_column = None
         self.perm=perm
+        self.color_list = color_list
         self._check_input()
-       
+
         if plot:
             self._perform_differential_expression_analysis()
             self._annotate_result_df()
@@ -57,8 +78,8 @@ class VolcanoPlot(PlotUtils):
         """
         update attributes using dict
         """
-        for key,value in updated_attributes.items():
-            setattr(self,key,value)
+        for key, value in updated_attributes.items():
+            setattr(self, key, value)
 
     @ignore_warning(UserWarning)
     @ignore_warning(RuntimeWarning)
@@ -83,42 +104,53 @@ class VolcanoPlot(PlotUtils):
         
         elif self.method == "sam":
             self._sam()
-        
-        # elif self.method == "Multi Covariates":
-        #    raise NotImplementedError
 
         else:
             raise ValueError(
                 f"{self.method} is not available."
-                + "Please select from 'ttest', 'sam' or 'anova' for anova with follow up tukey or 'wald' for wald-test."
+                + "Please select from 'ttest', 'sam', 'paired-ttest' or 'anova' for anova with follow up tukey or 'wald' for wald-test."
             )
 
     @lru_cache(maxsize=20)
     def _sam_calculate_fdr_line(self):
         from alphastats.multicova import multicova
 
-        self.fdr_line= multicova.get_fdr_line(
-                t_limit=self.tlim_ttest,
-                s0=0.05,
-                n_x=len(list(self.dataset.metadata[self.dataset.metadata[self.column]==self.group1][self.dataset.sample])),
-                n_y=len(list(self.dataset.metadata[self.dataset.metadata[self.column]==self.group2][self.dataset.sample])),
-                fc_s = np.arange(0,np.max(np.abs(self.res.log2fc)),np.max(np.abs(self.res.log2fc))/200),
-                s_s = np.arange(0.005, 6, 0.0025),
-                plot=False
-            )
-    
+        self.fdr_line = multicova.get_fdr_line(
+            t_limit=self.tlim_ttest,
+            s0=0.05,
+            n_x=len(
+                list(
+                    self.dataset.metadata[
+                        self.dataset.metadata[self.column] == self.group1
+                    ][self.dataset.sample]
+                )
+            ),
+            n_y=len(
+                list(
+                    self.dataset.metadata[
+                        self.dataset.metadata[self.column] == self.group2
+                    ][self.dataset.sample]
+                )
+            ),
+            fc_s=np.arange(
+                0,
+                np.max(np.abs(self.res.log2fc)),
+                np.max(np.abs(self.res.log2fc)) / 200,
+            ),
+            s_s=np.arange(0.005, 6, 0.0025),
+            plot=False,
+        )
+
     @lru_cache(maxsize=20)
     def _sam(self):
         from alphastats.multicova import multicova
 
-        print(
-            "Calculating t-test and permuation based FDR (SAM)... "
-        )
+        print("Calculating t-test and permuation based FDR (SAM)... ")
 
         transposed = self.dataset.mat.transpose()
 
         if self.dataset.preprocessing_info["Normalization"] is None:
-             # needs to be lpog2 transformed for fold change calculations
+            # needs to be lpog2 transformed for fold change calculations
             transposed = transposed.transform(lambda x: np.log2(x))
 
         transposed[self.dataset.index_column] = transposed.index
@@ -126,23 +158,40 @@ class VolcanoPlot(PlotUtils):
 
         res_ttest, tlim_ttest = multicova.perform_ttest_analysis(
             transposed,
-            c1 =list(self.dataset.metadata[self.dataset.metadata[self.column]==self.group1][self.dataset.sample]),                                      
-            c2 =list(self.dataset.metadata[self.dataset.metadata[self.column]==self.group2][self.dataset.sample]), 
-            s0=0.05, 
+            c1=list(
+                self.dataset.metadata[
+                    self.dataset.metadata[self.column] == self.group1
+                ][self.dataset.sample]
+            ),
+            c2=list(
+                self.dataset.metadata[
+                    self.dataset.metadata[self.column] == self.group2
+                ][self.dataset.sample]
+            ),
+            s0=0.05,
             n_perm=self.perm,
             fdr=self.fdr,
             id_col=self.dataset.index_column,
-            parallelize=True
+            parallelize=True,
         )
-        
-        fdr_column = "FDR"  + str(int(self.fdr*100)) + "%"
-        self.res = res_ttest[[self.dataset.index_column, 'fc', 'tval', 'pval', 'tval_s0', 'pval_s0', 'qval']]
+
+        fdr_column = "FDR" + str(int(self.fdr * 100)) + "%"
+        self.res = res_ttest[
+            [
+                self.dataset.index_column,
+                "fc",
+                "tval",
+                "pval",
+                "tval_s0",
+                "pval_s0",
+                "qval",
+            ]
+        ]
         self.res["log2fc"] = res_ttest["fc"]
         self.res["FDR"] = res_ttest[fdr_column]
         self.tlim_ttest = tlim_ttest
         self.pvalue_column = "pval"
 
-
     @lru_cache(maxsize=20)
     def _wald(self):
 
@@ -214,7 +263,9 @@ class VolcanoPlot(PlotUtils):
         ][self.dataset.sample].tolist()
 
         mat_transpose = self.dataset.mat.transpose()
-        fc = self.dataset._calculate_foldchange(mat_transpose, group1_samples, group2_samples)
+        fc = self.dataset._calculate_foldchange(
+            mat_transpose, group1_samples, group2_samples
+        )
 
         #  check how column is ordered
         self.pvalue_column = self.group1 + " vs. " + self.group2 + " Tukey Test"
@@ -222,7 +273,9 @@ class VolcanoPlot(PlotUtils):
         if self.pvalue_column not in result_df.columns:
             self.pvalue_column = self.group2 + " vs. " + self.group1 + " Tukey Test"
 
-        self.res = result_df.reset_index().merge(fc.reset_index(), on=self.dataset.index_column)
+        self.res = result_df.reset_index().merge(
+            fc.reset_index(), on=self.dataset.index_column
+        )
 
     def _add_hover_data_columns(self):
         # additional labeling with gene names
@@ -231,29 +284,32 @@ class VolcanoPlot(PlotUtils):
         if self.dataset.gene_names is not None:
             self.res = pd.merge(
                 self.res,
-                self.dataset.rawinput[[self.dataset.gene_names, self.dataset.index_column]],
+                self.dataset.rawinput[
+                    [self.dataset.gene_names, self.dataset.index_column]
+                ],
                 on=self.dataset.index_column,
                 how="left",
             )
             self.hover_data.append(self.dataset.gene_names)
 
-
     def _annotate_result_df(self):
         """
         convert pvalue to log10
         add color labels for up and down regulates
         """
-        self.res = self.res[(self.res["log2fc"] < 10) & (self.res["log2fc"] > -10)]
+        self.res = self.res[(self.res["log2fc"] < 20) & (self.res["log2fc"] > -20)]
         self.res["-log10(p-value)"] = -np.log10(self.res[self.pvalue_column])
-        
+
         self.alpha = -np.log10(self.alpha)
         # add color variable to plot
 
         if self.method != "sam":
-        
+
             condition = [
-                (self.res["log2fc"] < -self.min_fc) & (self.res["-log10(p-value)"] > self.alpha),
-                (self.res["log2fc"] > self.min_fc) & (self.res["-log10(p-value)"] > self.alpha),
+                (self.res["log2fc"] < -self.min_fc)
+                & (self.res["-log10(p-value)"] > self.alpha),
+                (self.res["log2fc"] > self.min_fc)
+                & (self.res["-log10(p-value)"] > self.alpha),
             ]
 
         else:
@@ -263,11 +319,16 @@ class VolcanoPlot(PlotUtils):
                 (self.res["log2fc"] > 0) & (self.res["FDR"] == "sig"),
             ]
 
-
         value = ["down", "up"]
+
         self.res["color"] = np.select(condition, value, default="non_sig")   
+
+        if len(self.color_list) > 0:
+            self.res["color"] = np.where(self.res[self.dataset.index_column].isin(self.color_list), 
+                                          "color", "no_color")   
         
 
+
     def _add_labels_plot(self):
         """
         add gene names as hover data if they are given
@@ -312,21 +373,35 @@ class VolcanoPlot(PlotUtils):
         Draw fdr line if SAM was applied
         """
         self._sam_calculate_fdr_line()
+
+        self.plot.add_trace(
+            go.Scatter(
+                x=self.fdr_line[self.fdr_line.fc_s > 0].fc_s,
+                y=-np.log10(self.fdr_line[self.fdr_line.fc_s > 0].pvals),
+                line_color="black",
+                line_shape="spline",
+                showlegend=False,
+            )
+        )
+        self.plot.add_trace(
+            go.Scatter(
+                x=self.fdr_line[self.fdr_line.fc_s < 0].fc_s,
+                y=-np.log10(self.fdr_line[self.fdr_line.fc_s < 0].pvals),
+                line_color="black",
+                line_shape="spline",
+                showlegend=False,
+            )
+        )
+    
+    def _color_data_points(self):
+         # update coloring
+        if len(self.color_list) == 0:
+            color_dict = {"non_sig": "#404040", "up": "#B65EAF", "down": "#009599"}
+    
+        else:
+            color_dict = {"no_color": "#404040", "color": "#B65EAF"}
         
-        self.plot.add_trace(go.Scatter(
-            x=self.fdr_line[self.fdr_line.fc_s > 0].fc_s,
-            y=-np.log10(self.fdr_line[self.fdr_line.fc_s > 0].pvals),
-            line_color="black",
-            line_shape='spline',
-            showlegend=False)
-        )
-        self.plot.add_trace(go.Scatter(
-            x=self.fdr_line[self.fdr_line.fc_s < 0].fc_s,
-            y=-np.log10(self.fdr_line[self.fdr_line.fc_s < 0].pvals),
-            line_color="black",
-            line_shape='spline',
-            showlegend=False)
-        )
+        self.plot = self._update_colors_plotly(self.plot, color_dict=color_dict)
 
 
     def _plot(self):
@@ -336,15 +411,15 @@ class VolcanoPlot(PlotUtils):
             y="-log10(p-value)",
             color="color",
             hover_data=self.hover_data,
+            template= "simple_white+alphastats_colors"
         )
-        
+
         # update coloring
-        color_dict = {"non_sig": "#404040", "up": "#B65EAF", "down": "#009599"}
-        self.plot = self._update_colors_plotly(self.plot, color_dict=color_dict)
+        self._color_data_points()
 
         if self.labels:
             self._add_labels_plot()
-        
+
         if self.draw_line:
             if self.method == "sam":
                 self._draw_fdr_line()
@@ -357,11 +432,8 @@ class VolcanoPlot(PlotUtils):
         #  save plotting data in figure object
         self.plot = plotly_object(self.plot)
         self.plot = self._update_figure_attributes(
-            figure_object=self.plot, 
-            plotting_data=self.res, 
-            preprocessing_info=self.dataset.preprocessing_info, 
-            method=self.method
+            figure_object=self.plot,
+            plotting_data=self.res,
+            preprocessing_info=self.dataset.preprocessing_info,
+            method=self.method,
         )
-    
-
-   
